import type { MarkdownPostProcessorContext } from "obsidian";
import { mount, unmount, type Component, type ComponentProps } from "svelte";
import { MarkdownRenderChild } from "obsidian";

/**
 * Renders a svelte component as a code block processor.
 * @param component the svelte component to render.
 * @param props properties forwarded to the component.
 * @param stateProvider an optional provider that handles state & state updates of the code block processor.
 */
export function renderCodeBlockProcessor<C extends Component>(
	component: C,
	props: ComponentProps<C>
) {
	return (
		source: string,
		containerEl: HTMLElement,
		ctx: MarkdownPostProcessorContext
	) => {
		const svelteComponent = mount(component, {
			target: containerEl,
			props,
		});

		class UnloadSvelteComponent extends MarkdownRenderChild {
			onunload() {
				unmount(svelteComponent);
			}
		}

		ctx.addChild(new UnloadSvelteComponent(containerEl));
	};
}
