import GoodreadsSearch from "@components/GoodreadsSearch.svelte";
import { type SearchResult } from "@data-sources/goodreads";
import { Event, EventEmitter } from "@utils/event";
import { App } from "obsidian";
import { SvelteModal } from "./svelte-modal";

export class SearchEvent extends Event {
	constructor(
		public readonly query: string,
		public readonly results: SearchResult[]
	) {
		super();
	}
}

export class ErrorEvent extends Event {
	constructor(public readonly error: Error) {
		super();
	}
}

interface GoodreadsSearchModalEventMap {
	search: SearchEvent;
	error: ErrorEvent;
}

export class GoodreadsSearchModal extends EventEmitter<
	GoodreadsSearchModalEventMap,
	typeof SvelteModal<typeof GoodreadsSearch>
>(SvelteModal) {
	constructor(app: App) {
		super(app, GoodreadsSearch, {
			props: {
				onError: (error: Error) => {
					this.emit("error", new ErrorEvent(error));
				},
				onSearch: (query: string, results: SearchResult[]) => {
					this.emit("search", new SearchEvent(query, results));
				},
			},
		});
	}

	static createAndOpen(app: App): Promise<SearchResult[]> {
		return new Promise((resolve, reject) => {
			const modal = new GoodreadsSearchModal(app);
			modal.once("search", (event: SearchEvent) => {
				modal.close();
				resolve(event.results);
			});
			modal.once("error", (event: ErrorEvent) => {
				modal.close();
				reject(event.error);
			});
			modal.open();
		});
	}
}
